/* Generated */

#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlAutoCloseTag(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlCreateFileParserCtxt(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlCreateMemoryParserCtxt(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlCreatePushParser(PyObject *self, PyObject *args);
#endif
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlCtxtReadDoc(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlCtxtReadFd(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlCtxtReadFile(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlCtxtReadMemory(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlCtxtReset(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlCtxtSetOptions(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlCtxtUseOptions(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlDefaultSAXHandlerInit(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED) && defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_htmlDocContentDumpFormatOutput(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) && defined(LIBXML_OUTPUT_ENABLED) */
#if defined(LIBXML_HTML_ENABLED) && defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_htmlDocContentDumpOutput(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) && defined(LIBXML_OUTPUT_ENABLED) */
#if defined(LIBXML_HTML_ENABLED) && defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_htmlDocDump(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) && defined(LIBXML_OUTPUT_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlFreeParserCtxt(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlGetMetaEncoding(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlHandleOmittedElem(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlInitAutoClose(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlIsAutoClosed(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlIsBooleanAttr(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlIsScriptAttribute(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlNewDoc(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlNewDocNoDtD(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlNewParserCtxt(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED) && defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_htmlNodeDumpFile(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) && defined(LIBXML_OUTPUT_ENABLED) */
#if defined(LIBXML_HTML_ENABLED) && defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_htmlNodeDumpFileFormat(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) && defined(LIBXML_OUTPUT_ENABLED) */
#if defined(LIBXML_HTML_ENABLED) && defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_htmlNodeDumpFormatOutput(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) && defined(LIBXML_OUTPUT_ENABLED) */
#if defined(LIBXML_HTML_ENABLED) && defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_htmlNodeDumpOutput(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) && defined(LIBXML_OUTPUT_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlParseCharRef(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED) && defined(LIBXML_PUSH_ENABLED)
PyObject * libxml_htmlParseChunk(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) && defined(LIBXML_PUSH_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlParseDoc(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlParseDocument(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlParseElement(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlParseFile(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlReadDoc(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlReadFd(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlReadFile(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlReadMemory(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlSAXParseFile(PyObject *self, PyObject *args);
#endif
#if defined(LIBXML_HTML_ENABLED) && defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_htmlSaveFile(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) && defined(LIBXML_OUTPUT_ENABLED) */
#if defined(LIBXML_HTML_ENABLED) && defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_htmlSaveFileEnc(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) && defined(LIBXML_OUTPUT_ENABLED) */
#if defined(LIBXML_HTML_ENABLED) && defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_htmlSaveFileFormat(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) && defined(LIBXML_OUTPUT_ENABLED) */
#if defined(LIBXML_HTML_ENABLED)
PyObject * libxml_htmlSetMetaEncoding(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_HTML_ENABLED) */
#if defined(LIBXML_CATALOG_ENABLED)
PyObject * libxml_xmlACatalogAdd(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_CATALOG_ENABLED) */
#if defined(LIBXML_CATALOG_ENABLED) && defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_xmlACatalogDump(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_CATALOG_ENABLED) && defined(LIBXML_OUTPUT_ENABLED) */
#if defined(LIBXML_CATALOG_ENABLED)
PyObject * libxml_xmlACatalogRemove(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_CATALOG_ENABLED) */
#if defined(LIBXML_CATALOG_ENABLED)
PyObject * libxml_xmlACatalogResolve(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_CATALOG_ENABLED) */
#if defined(LIBXML_CATALOG_ENABLED)
PyObject * libxml_xmlACatalogResolvePublic(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_CATALOG_ENABLED) */
#if defined(LIBXML_CATALOG_ENABLED)
PyObject * libxml_xmlACatalogResolveSystem(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_CATALOG_ENABLED) */
#if defined(LIBXML_CATALOG_ENABLED)
PyObject * libxml_xmlACatalogResolveURI(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_CATALOG_ENABLED) */
PyObject * libxml_xmlAddChild(PyObject *self, PyObject *args);
PyObject * libxml_xmlAddChildList(PyObject *self, PyObject *args);
PyObject * libxml_xmlAddDocEntity(PyObject *self, PyObject *args);
PyObject * libxml_xmlAddDtdEntity(PyObject *self, PyObject *args);
PyObject * libxml_xmlAddEncodingAlias(PyObject *self, PyObject *args);
PyObject * libxml_xmlAddIDSafe(PyObject *self, PyObject *args);
PyObject * libxml_xmlAddNextSibling(PyObject *self, PyObject *args);
PyObject * libxml_xmlAddPrevSibling(PyObject *self, PyObject *args);
PyObject * libxml_xmlAddSibling(PyObject *self, PyObject *args);
PyObject * libxml_xmlBuildQName(PyObject *self, PyObject *args);
PyObject * libxml_xmlBuildRelativeURI(PyObject *self, PyObject *args);
PyObject * libxml_xmlBuildURI(PyObject *self, PyObject *args);
PyObject * libxml_xmlByteConsumed(PyObject *self, PyObject *args);
PyObject * libxml_xmlCanonicPath(PyObject *self, PyObject *args);
#if defined(LIBXML_CATALOG_ENABLED)
PyObject * libxml_xmlCatalogAdd(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_CATALOG_ENABLED) */
#if defined(LIBXML_CATALOG_ENABLED)
PyObject * libxml_xmlCatalogCleanup(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_CATALOG_ENABLED) */
#if defined(LIBXML_SGML_CATALOG_ENABLED)
PyObject * libxml_xmlCatalogConvert(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SGML_CATALOG_ENABLED) */
#if defined(LIBXML_CATALOG_ENABLED) && defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_xmlCatalogDump(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_CATALOG_ENABLED) && defined(LIBXML_OUTPUT_ENABLED) */
#if defined(LIBXML_CATALOG_ENABLED)
PyObject * libxml_xmlCatalogGetPublic(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_CATALOG_ENABLED) */
#if defined(LIBXML_CATALOG_ENABLED)
PyObject * libxml_xmlCatalogGetSystem(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_CATALOG_ENABLED) */
#if defined(LIBXML_CATALOG_ENABLED)
PyObject * libxml_xmlCatalogIsEmpty(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_CATALOG_ENABLED) */
#if defined(LIBXML_CATALOG_ENABLED)
PyObject * libxml_xmlCatalogRemove(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_CATALOG_ENABLED) */
#if defined(LIBXML_CATALOG_ENABLED)
PyObject * libxml_xmlCatalogResolve(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_CATALOG_ENABLED) */
#if defined(LIBXML_CATALOG_ENABLED)
PyObject * libxml_xmlCatalogResolvePublic(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_CATALOG_ENABLED) */
#if defined(LIBXML_CATALOG_ENABLED)
PyObject * libxml_xmlCatalogResolveSystem(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_CATALOG_ENABLED) */
#if defined(LIBXML_CATALOG_ENABLED)
PyObject * libxml_xmlCatalogResolveURI(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_CATALOG_ENABLED) */
#if defined(LIBXML_CATALOG_ENABLED)
PyObject * libxml_xmlCatalogSetDebug(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_CATALOG_ENABLED) */
PyObject * libxml_xmlCharStrdup(PyObject *self, PyObject *args);
PyObject * libxml_xmlCharStrndup(PyObject *self, PyObject *args);
PyObject * libxml_xmlCheckFilename(PyObject *self, PyObject *args);
PyObject * libxml_xmlCheckLanguageID(PyObject *self, PyObject *args);
PyObject * libxml_xmlCheckUTF8(PyObject *self, PyObject *args);
PyObject * libxml_xmlCleanupCharEncodingHandlers(PyObject *self, PyObject *args);
PyObject * libxml_xmlCleanupEncodingAliases(PyObject *self, PyObject *args);
PyObject * libxml_xmlCleanupGlobals(PyObject *self, PyObject *args);
PyObject * libxml_xmlCleanupInputCallbacks(PyObject *self, PyObject *args);
#if defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_xmlCleanupOutputCallbacks(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_OUTPUT_ENABLED) */
PyObject * libxml_xmlClearParserCtxt(PyObject *self, PyObject *args);
#if defined(LIBXML_SGML_CATALOG_ENABLED)
PyObject * libxml_xmlConvertSGMLCatalog(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SGML_CATALOG_ENABLED) */
PyObject * libxml_xmlCopyChar(PyObject *self, PyObject *args);
PyObject * libxml_xmlCopyCharMultiByte(PyObject *self, PyObject *args);
PyObject * libxml_xmlCopyDoc(PyObject *self, PyObject *args);
PyObject * libxml_xmlCopyDtd(PyObject *self, PyObject *args);
PyObject * libxml_xmlCopyError(PyObject *self, PyObject *args);
PyObject * libxml_xmlCopyNamespace(PyObject *self, PyObject *args);
PyObject * libxml_xmlCopyNamespaceList(PyObject *self, PyObject *args);
PyObject * libxml_xmlCopyNode(PyObject *self, PyObject *args);
PyObject * libxml_xmlCopyNodeList(PyObject *self, PyObject *args);
PyObject * libxml_xmlCopyProp(PyObject *self, PyObject *args);
PyObject * libxml_xmlCopyPropList(PyObject *self, PyObject *args);
PyObject * libxml_xmlCreateDocParserCtxt(PyObject *self, PyObject *args);
PyObject * libxml_xmlCreateEntityParserCtxt(PyObject *self, PyObject *args);
PyObject * libxml_xmlCreateFileParserCtxt(PyObject *self, PyObject *args);
PyObject * libxml_xmlCreateInputBuffer(PyObject *self, PyObject *args);
PyObject * libxml_xmlCreateIntSubset(PyObject *self, PyObject *args);
PyObject * libxml_xmlCreateMemoryParserCtxt(PyObject *self, PyObject *args);
PyObject * libxml_xmlCreateOutputBuffer(PyObject *self, PyObject *args);
PyObject * libxml_xmlCreatePushParser(PyObject *self, PyObject *args);
PyObject * libxml_xmlCreateURI(PyObject *self, PyObject *args);
PyObject * libxml_xmlCreateURLParserCtxt(PyObject *self, PyObject *args);
PyObject * libxml_xmlCtxtErrMemory(PyObject *self, PyObject *args);
PyObject * libxml_xmlCtxtGetDeclaredEncoding(PyObject *self, PyObject *args);
PyObject * libxml_xmlCtxtGetDocument(PyObject *self, PyObject *args);
PyObject * libxml_xmlCtxtGetNode(PyObject *self, PyObject *args);
PyObject * libxml_xmlCtxtGetOptions(PyObject *self, PyObject *args);
PyObject * libxml_xmlCtxtGetStandalone(PyObject *self, PyObject *args);
#if defined(LIBXML_VALID_ENABLED)
PyObject * libxml_xmlCtxtGetValidCtxt(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_VALID_ENABLED) */
PyObject * libxml_xmlCtxtGetVersion(PyObject *self, PyObject *args);
PyObject * libxml_xmlCtxtIsHtml(PyObject *self, PyObject *args);
PyObject * libxml_xmlCtxtIsInSubset(PyObject *self, PyObject *args);
PyObject * libxml_xmlCtxtIsStopped(PyObject *self, PyObject *args);
PyObject * libxml_xmlCtxtReadDoc(PyObject *self, PyObject *args);
PyObject * libxml_xmlCtxtReadFd(PyObject *self, PyObject *args);
PyObject * libxml_xmlCtxtReadFile(PyObject *self, PyObject *args);
PyObject * libxml_xmlCtxtReadMemory(PyObject *self, PyObject *args);
PyObject * libxml_xmlCtxtReset(PyObject *self, PyObject *args);
PyObject * libxml_xmlCtxtResetPush(PyObject *self, PyObject *args);
PyObject * libxml_xmlCtxtSetOptions(PyObject *self, PyObject *args);
PyObject * libxml_xmlCtxtUseOptions(PyObject *self, PyObject *args);
#if defined(LIBXML_VALID_ENABLED)
PyObject * libxml_xmlCtxtValidateDocument(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_VALID_ENABLED) */
#if defined(LIBXML_VALID_ENABLED)
PyObject * libxml_xmlCtxtValidateDtd(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_VALID_ENABLED) */
#if defined(LIBXML_DEBUG_ENABLED)
PyObject * libxml_xmlDebugCheckDocument(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_DEBUG_ENABLED) */
#if defined(LIBXML_DEBUG_ENABLED)
PyObject * libxml_xmlDebugDumpAttr(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_DEBUG_ENABLED) */
#if defined(LIBXML_DEBUG_ENABLED)
PyObject * libxml_xmlDebugDumpAttrList(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_DEBUG_ENABLED) */
#if defined(LIBXML_DEBUG_ENABLED)
PyObject * libxml_xmlDebugDumpDTD(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_DEBUG_ENABLED) */
#if defined(LIBXML_DEBUG_ENABLED)
PyObject * libxml_xmlDebugDumpDocument(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_DEBUG_ENABLED) */
#if defined(LIBXML_DEBUG_ENABLED)
PyObject * libxml_xmlDebugDumpDocumentHead(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_DEBUG_ENABLED) */
#if defined(LIBXML_DEBUG_ENABLED)
PyObject * libxml_xmlDebugDumpEntities(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_DEBUG_ENABLED) */
#if defined(LIBXML_DEBUG_ENABLED)
PyObject * libxml_xmlDebugDumpNode(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_DEBUG_ENABLED) */
#if defined(LIBXML_DEBUG_ENABLED)
PyObject * libxml_xmlDebugDumpNodeList(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_DEBUG_ENABLED) */
#if defined(LIBXML_DEBUG_ENABLED)
PyObject * libxml_xmlDebugDumpOneNode(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_DEBUG_ENABLED) */
#if defined(LIBXML_DEBUG_ENABLED)
PyObject * libxml_xmlDebugDumpString(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_DEBUG_ENABLED) */
PyObject * libxml_xmlDebugMemory(PyObject *self, PyObject *args);
PyObject * libxml_xmlDefaultSAXHandlerInit(PyObject *self, PyObject *args);
PyObject * libxml_xmlDelEncodingAlias(PyObject *self, PyObject *args);
PyObject * libxml_xmlDictCleanup(PyObject *self, PyObject *args);
PyObject * libxml_xmlDocCopyNode(PyObject *self, PyObject *args);
PyObject * libxml_xmlDocCopyNodeList(PyObject *self, PyObject *args);
#if defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_xmlDocDump(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_OUTPUT_ENABLED) */
#if defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_xmlDocFormatDump(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_OUTPUT_ENABLED) */
PyObject * libxml_xmlDocGetRootElement(PyObject *self, PyObject *args);
PyObject * libxml_xmlDocSetRootElement(PyObject *self, PyObject *args);
#if defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_xmlElemDump(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_OUTPUT_ENABLED) */
PyObject * libxml_xmlEncodeEntitiesReentrant(PyObject *self, PyObject *args);
PyObject * libxml_xmlEncodeSpecialChars(PyObject *self, PyObject *args);
PyObject * libxml_xmlErrorGetCode(PyObject *self, PyObject *args);
PyObject * libxml_xmlErrorGetDomain(PyObject *self, PyObject *args);
PyObject * libxml_xmlErrorGetFile(PyObject *self, PyObject *args);
PyObject * libxml_xmlErrorGetLevel(PyObject *self, PyObject *args);
PyObject * libxml_xmlErrorGetLine(PyObject *self, PyObject *args);
PyObject * libxml_xmlErrorGetMessage(PyObject *self, PyObject *args);
PyObject * libxml_xmlFileMatch(PyObject *self, PyObject *args);
PyObject * libxml_xmlFirstElementChild(PyObject *self, PyObject *args);
#if defined(LIBXML_CATALOG_ENABLED)
PyObject * libxml_xmlFreeCatalog(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_CATALOG_ENABLED) */
PyObject * libxml_xmlFreeDoc(PyObject *self, PyObject *args);
PyObject * libxml_xmlFreeDtd(PyObject *self, PyObject *args);
PyObject * libxml_xmlFreeEntity(PyObject *self, PyObject *args);
PyObject * libxml_xmlFreeNode(PyObject *self, PyObject *args);
PyObject * libxml_xmlFreeNodeList(PyObject *self, PyObject *args);
PyObject * libxml_xmlFreeNs(PyObject *self, PyObject *args);
PyObject * libxml_xmlFreeNsList(PyObject *self, PyObject *args);
PyObject * libxml_xmlFreeParserInputBuffer(PyObject *self, PyObject *args);
PyObject * libxml_xmlFreeProp(PyObject *self, PyObject *args);
PyObject * libxml_xmlFreePropList(PyObject *self, PyObject *args);
PyObject * libxml_xmlFreeURI(PyObject *self, PyObject *args);
PyObject * libxml_xmlGetCompressMode(PyObject *self, PyObject *args);
PyObject * libxml_xmlGetDocCompressMode(PyObject *self, PyObject *args);
PyObject * libxml_xmlGetDocEntity(PyObject *self, PyObject *args);
PyObject * libxml_xmlGetDtdAttrDesc(PyObject *self, PyObject *args);
PyObject * libxml_xmlGetDtdElementDesc(PyObject *self, PyObject *args);
PyObject * libxml_xmlGetDtdEntity(PyObject *self, PyObject *args);
PyObject * libxml_xmlGetDtdQAttrDesc(PyObject *self, PyObject *args);
PyObject * libxml_xmlGetDtdQElementDesc(PyObject *self, PyObject *args);
PyObject * libxml_xmlGetEncodingAlias(PyObject *self, PyObject *args);
PyObject * libxml_xmlGetID(PyObject *self, PyObject *args);
PyObject * libxml_xmlGetIntSubset(PyObject *self, PyObject *args);
PyObject * libxml_xmlGetLastChild(PyObject *self, PyObject *args);
PyObject * libxml_xmlGetLastError(PyObject *self, PyObject *args);
PyObject * libxml_xmlGetLineNo(PyObject *self, PyObject *args);
PyObject * libxml_xmlGetNoNsProp(PyObject *self, PyObject *args);
PyObject * libxml_xmlGetNodePath(PyObject *self, PyObject *args);
PyObject * libxml_xmlGetNsProp(PyObject *self, PyObject *args);
PyObject * libxml_xmlGetParameterEntity(PyObject *self, PyObject *args);
PyObject * libxml_xmlGetPredefinedEntity(PyObject *self, PyObject *args);
PyObject * libxml_xmlGetProp(PyObject *self, PyObject *args);
PyObject * libxml_xmlHasNsProp(PyObject *self, PyObject *args);
PyObject * libxml_xmlHasProp(PyObject *self, PyObject *args);
PyObject * libxml_xmlInitCharEncodingHandlers(PyObject *self, PyObject *args);
PyObject * libxml_xmlInitGlobals(PyObject *self, PyObject *args);
PyObject * libxml_xmlInitParser(PyObject *self, PyObject *args);
PyObject * libxml_xmlInitParserCtxt(PyObject *self, PyObject *args);
#if defined(LIBXML_CATALOG_ENABLED)
PyObject * libxml_xmlInitializeCatalog(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_CATALOG_ENABLED) */
PyObject * libxml_xmlInitializeDict(PyObject *self, PyObject *args);
PyObject * libxml_xmlIsBaseChar(PyObject *self, PyObject *args);
PyObject * libxml_xmlIsBlank(PyObject *self, PyObject *args);
PyObject * libxml_xmlIsBlankNode(PyObject *self, PyObject *args);
PyObject * libxml_xmlIsChar(PyObject *self, PyObject *args);
PyObject * libxml_xmlIsCombining(PyObject *self, PyObject *args);
PyObject * libxml_xmlIsDigit(PyObject *self, PyObject *args);
PyObject * libxml_xmlIsExtender(PyObject *self, PyObject *args);
PyObject * libxml_xmlIsID(PyObject *self, PyObject *args);
PyObject * libxml_xmlIsIdeographic(PyObject *self, PyObject *args);
PyObject * libxml_xmlIsLetter(PyObject *self, PyObject *args);
PyObject * libxml_xmlIsMixedElement(PyObject *self, PyObject *args);
PyObject * libxml_xmlIsPubidChar(PyObject *self, PyObject *args);
PyObject * libxml_xmlIsRef(PyObject *self, PyObject *args);
PyObject * libxml_xmlIsXHTML(PyObject *self, PyObject *args);
PyObject * libxml_xmlKeepBlanksDefault(PyObject *self, PyObject *args);
PyObject * libxml_xmlLastElementChild(PyObject *self, PyObject *args);
PyObject * libxml_xmlLineNumbersDefault(PyObject *self, PyObject *args);
#if defined(LIBXML_CATALOG_ENABLED)
PyObject * libxml_xmlLoadACatalog(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_CATALOG_ENABLED) */
#if defined(LIBXML_CATALOG_ENABLED)
PyObject * libxml_xmlLoadCatalog(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_CATALOG_ENABLED) */
#if defined(LIBXML_CATALOG_ENABLED)
PyObject * libxml_xmlLoadCatalogs(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_CATALOG_ENABLED) */
#if defined(LIBXML_SGML_CATALOG_ENABLED)
PyObject * libxml_xmlLoadSGMLSuperCatalog(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SGML_CATALOG_ENABLED) */
PyObject * libxml_xmlMemoryUsed(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewCDataBlock(PyObject *self, PyObject *args);
#if defined(LIBXML_CATALOG_ENABLED)
PyObject * libxml_xmlNewCatalog(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_CATALOG_ENABLED) */
PyObject * libxml_xmlNewCharRef(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewChild(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewComment(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewDoc(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewDocComment(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewDocFragment(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewDocNode(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewDocNodeEatName(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewDocPI(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewDocProp(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewDocRawNode(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewDocText(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewDocTextLen(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewDtd(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewEntity(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewNode(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewNodeEatName(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewNs(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewNsProp(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewNsPropEatName(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewPI(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewParserCtxt(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewProp(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewReference(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewText(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewTextChild(PyObject *self, PyObject *args);
PyObject * libxml_xmlNewTextLen(PyObject *self, PyObject *args);
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlNewTextReader(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlNewTextReaderFilename(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_VALID_ENABLED)
PyObject * libxml_xmlNewValidCtxt(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_VALID_ENABLED) */
PyObject * libxml_xmlNextChar(PyObject *self, PyObject *args);
PyObject * libxml_xmlNextElementSibling(PyObject *self, PyObject *args);
PyObject * libxml_xmlNodeAddContent(PyObject *self, PyObject *args);
PyObject * libxml_xmlNodeAddContentLen(PyObject *self, PyObject *args);
#if defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_xmlNodeDumpOutput(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_OUTPUT_ENABLED) */
PyObject * libxml_xmlNodeGetBase(PyObject *self, PyObject *args);
PyObject * libxml_xmlNodeGetContent(PyObject *self, PyObject *args);
PyObject * libxml_xmlNodeGetLang(PyObject *self, PyObject *args);
PyObject * libxml_xmlNodeGetNs(PyObject *self, PyObject *args);
PyObject * libxml_xmlNodeGetNsDefs(PyObject *self, PyObject *args);
PyObject * libxml_xmlNodeGetSpacePreserve(PyObject *self, PyObject *args);
PyObject * libxml_xmlNodeIsText(PyObject *self, PyObject *args);
PyObject * libxml_xmlNodeListGetRawString(PyObject *self, PyObject *args);
PyObject * libxml_xmlNodeListGetString(PyObject *self, PyObject *args);
PyObject * libxml_xmlNodeSetBase(PyObject *self, PyObject *args);
PyObject * libxml_xmlNodeSetContent(PyObject *self, PyObject *args);
PyObject * libxml_xmlNodeSetContentLen(PyObject *self, PyObject *args);
PyObject * libxml_xmlNodeSetLang(PyObject *self, PyObject *args);
PyObject * libxml_xmlNodeSetName(PyObject *self, PyObject *args);
PyObject * libxml_xmlNodeSetSpacePreserve(PyObject *self, PyObject *args);
PyObject * libxml_xmlNormalizeURIPath(PyObject *self, PyObject *args);
PyObject * libxml_xmlNormalizeWindowsPath(PyObject *self, PyObject *args);
#if defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_xmlOutputBufferGetContent(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_OUTPUT_ENABLED) */
#if defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_xmlOutputBufferWrite(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_OUTPUT_ENABLED) */
#if defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_xmlOutputBufferWriteString(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_OUTPUT_ENABLED) */
PyObject * libxml_xmlParseAttValue(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseAttributeListDecl(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseCDSect(PyObject *self, PyObject *args);
#if defined(LIBXML_CATALOG_ENABLED)
PyObject * libxml_xmlParseCatalogFile(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_CATALOG_ENABLED) */
PyObject * libxml_xmlParseCharData(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseCharRef(PyObject *self, PyObject *args);
#if defined(LIBXML_PUSH_ENABLED)
PyObject * libxml_xmlParseChunk(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_PUSH_ENABLED) */
PyObject * libxml_xmlParseComment(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseContent(PyObject *self, PyObject *args);
#if defined(LIBXML_VALID_ENABLED)
PyObject * libxml_xmlParseDTD(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_VALID_ENABLED) */
#if defined(LIBXML_SAX1_ENABLED)
PyObject * libxml_xmlParseDoc(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SAX1_ENABLED) */
PyObject * libxml_xmlParseDocTypeDecl(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseDocument(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseElement(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseElementDecl(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseEncName(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseEncodingDecl(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseEndTag(PyObject *self, PyObject *args);
#if defined(LIBXML_SAX1_ENABLED)
PyObject * libxml_xmlParseEntity(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SAX1_ENABLED) */
PyObject * libxml_xmlParseEntityDecl(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseEntityRef(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseExtParsedEnt(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseExternalSubset(PyObject *self, PyObject *args);
#if defined(LIBXML_SAX1_ENABLED)
PyObject * libxml_xmlParseFile(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SAX1_ENABLED) */
PyObject * libxml_xmlParseMarkupDecl(PyObject *self, PyObject *args);
#if defined(LIBXML_SAX1_ENABLED)
PyObject * libxml_xmlParseMemory(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SAX1_ENABLED) */
PyObject * libxml_xmlParseMisc(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseName(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseNmtoken(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseNotationDecl(PyObject *self, PyObject *args);
PyObject * libxml_xmlParsePEReference(PyObject *self, PyObject *args);
PyObject * libxml_xmlParsePI(PyObject *self, PyObject *args);
PyObject * libxml_xmlParsePITarget(PyObject *self, PyObject *args);
PyObject * libxml_xmlParsePubidLiteral(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseReference(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseSDDecl(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseStartTag(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseSystemLiteral(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseTextDecl(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseURI(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseURIRaw(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseURIReference(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseVersionInfo(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseVersionNum(PyObject *self, PyObject *args);
PyObject * libxml_xmlParseXMLDecl(PyObject *self, PyObject *args);
PyObject * libxml_xmlParserGetDirectory(PyObject *self, PyObject *args);
PyObject * libxml_xmlParserGetDoc(PyObject *self, PyObject *args);
PyObject * libxml_xmlParserGetIsValid(PyObject *self, PyObject *args);
PyObject * libxml_xmlParserGetWellFormed(PyObject *self, PyObject *args);
PyObject * libxml_xmlParserHandlePEReference(PyObject *self, PyObject *args);
PyObject * libxml_xmlParserInputBufferGrow(PyObject *self, PyObject *args);
PyObject * libxml_xmlParserInputBufferPush(PyObject *self, PyObject *args);
PyObject * libxml_xmlParserInputBufferRead(PyObject *self, PyObject *args);
PyObject * libxml_xmlParserSetLineNumbers(PyObject *self, PyObject *args);
PyObject * libxml_xmlParserSetLoadSubset(PyObject *self, PyObject *args);
PyObject * libxml_xmlParserSetPedantic(PyObject *self, PyObject *args);
PyObject * libxml_xmlParserSetReplaceEntities(PyObject *self, PyObject *args);
PyObject * libxml_xmlParserSetValidate(PyObject *self, PyObject *args);
PyObject * libxml_xmlPathToURI(PyObject *self, PyObject *args);
PyObject * libxml_xmlPedanticParserDefault(PyObject *self, PyObject *args);
PyObject * libxml_xmlPopInput(PyObject *self, PyObject *args);
#if defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_xmlPopOutputCallbacks(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_OUTPUT_ENABLED) */
PyObject * libxml_xmlPreviousElementSibling(PyObject *self, PyObject *args);
PyObject * libxml_xmlPrintURI(PyObject *self, PyObject *args);
PyObject * libxml_xmlPythonCleanupParser(PyObject *self, PyObject *args);
PyObject * libxml_xmlReadDoc(PyObject *self, PyObject *args);
PyObject * libxml_xmlReadFd(PyObject *self, PyObject *args);
PyObject * libxml_xmlReadFile(PyObject *self, PyObject *args);
PyObject * libxml_xmlReadMemory(PyObject *self, PyObject *args);
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlReaderForDoc(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlReaderForFd(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlReaderForFile(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlReaderForMemory(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlReaderNewDoc(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlReaderNewFd(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlReaderNewFile(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlReaderNewMemory(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlReaderNewWalker(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlReaderWalker(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
PyObject * libxml_xmlReconciliateNs(PyObject *self, PyObject *args);
#if defined(LIBXML_SAX1_ENABLED)
PyObject * libxml_xmlRecoverDoc(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SAX1_ENABLED) */
#if defined(LIBXML_SAX1_ENABLED)
PyObject * libxml_xmlRecoverFile(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SAX1_ENABLED) */
#if defined(LIBXML_SAX1_ENABLED)
PyObject * libxml_xmlRecoverMemory(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SAX1_ENABLED) */
#if defined(LIBXML_REGEXP_ENABLED)
PyObject * libxml_xmlRegFreeRegexp(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_REGEXP_ENABLED) */
#if defined(LIBXML_REGEXP_ENABLED)
PyObject * libxml_xmlRegexpCompile(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_REGEXP_ENABLED) */
#if defined(LIBXML_REGEXP_ENABLED)
PyObject * libxml_xmlRegexpExec(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_REGEXP_ENABLED) */
#if defined(LIBXML_REGEXP_ENABLED)
PyObject * libxml_xmlRegexpIsDeterminist(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_REGEXP_ENABLED) */
#if defined(LIBXML_REGEXP_ENABLED)
PyObject * libxml_xmlRegexpPrint(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_REGEXP_ENABLED) */
PyObject * libxml_xmlRegisterDefaultInputCallbacks(PyObject *self, PyObject *args);
#if defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_xmlRegisterDefaultOutputCallbacks(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_OUTPUT_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlRegisterXPathFunction(PyObject *self, PyObject *args);
#endif
#if defined(LIBXML_RELAXNG_ENABLED)
PyObject * libxml_xmlRelaxNGCleanupTypes(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_RELAXNG_ENABLED) */
#if defined(LIBXML_RELAXNG_ENABLED) && defined(LIBXML_DEBUG_ENABLED)
PyObject * libxml_xmlRelaxNGDump(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_RELAXNG_ENABLED) && defined(LIBXML_DEBUG_ENABLED) */
#if defined(LIBXML_RELAXNG_ENABLED) && defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_xmlRelaxNGDumpTree(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_RELAXNG_ENABLED) && defined(LIBXML_OUTPUT_ENABLED) */
#if defined(LIBXML_RELAXNG_ENABLED)
PyObject * libxml_xmlRelaxNGFree(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_RELAXNG_ENABLED) */
#if defined(LIBXML_RELAXNG_ENABLED)
PyObject * libxml_xmlRelaxNGFreeParserCtxt(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_RELAXNG_ENABLED) */
#if defined(LIBXML_RELAXNG_ENABLED)
PyObject * libxml_xmlRelaxNGInitTypes(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_RELAXNG_ENABLED) */
#if defined(LIBXML_RELAXNG_ENABLED)
PyObject * libxml_xmlRelaxNGNewDocParserCtxt(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_RELAXNG_ENABLED) */
#if defined(LIBXML_RELAXNG_ENABLED)
PyObject * libxml_xmlRelaxNGNewMemParserCtxt(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_RELAXNG_ENABLED) */
#if defined(LIBXML_RELAXNG_ENABLED)
PyObject * libxml_xmlRelaxNGNewParserCtxt(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_RELAXNG_ENABLED) */
#if defined(LIBXML_RELAXNG_ENABLED)
PyObject * libxml_xmlRelaxNGNewValidCtxt(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_RELAXNG_ENABLED) */
#if defined(LIBXML_RELAXNG_ENABLED)
PyObject * libxml_xmlRelaxNGParse(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_RELAXNG_ENABLED) */
#if defined(LIBXML_RELAXNG_ENABLED)
PyObject * libxml_xmlRelaxNGValidCtxtClearErrors(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_RELAXNG_ENABLED) */
#if defined(LIBXML_RELAXNG_ENABLED)
PyObject * libxml_xmlRelaxNGValidateDoc(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_RELAXNG_ENABLED) */
#if defined(LIBXML_RELAXNG_ENABLED)
PyObject * libxml_xmlRelaxNGValidateFullElement(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_RELAXNG_ENABLED) */
#if defined(LIBXML_RELAXNG_ENABLED)
PyObject * libxml_xmlRelaxNGValidatePopElement(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_RELAXNG_ENABLED) */
#if defined(LIBXML_RELAXNG_ENABLED)
PyObject * libxml_xmlRelaxNGValidatePushCData(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_RELAXNG_ENABLED) */
#if defined(LIBXML_RELAXNG_ENABLED)
PyObject * libxml_xmlRelaxNGValidatePushElement(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_RELAXNG_ENABLED) */
#if defined(LIBXML_RELAXNG_ENABLED)
PyObject * libxml_xmlRelaxParserSetFlag(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_RELAXNG_ENABLED) */
#if defined(LIBXML_RELAXNG_ENABLED)
PyObject * libxml_xmlRelaxParserSetIncLImit(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_RELAXNG_ENABLED) */
PyObject * libxml_xmlRemoveID(PyObject *self, PyObject *args);
PyObject * libxml_xmlRemoveProp(PyObject *self, PyObject *args);
PyObject * libxml_xmlRemoveRef(PyObject *self, PyObject *args);
PyObject * libxml_xmlReplaceNode(PyObject *self, PyObject *args);
PyObject * libxml_xmlResetError(PyObject *self, PyObject *args);
PyObject * libxml_xmlResetLastError(PyObject *self, PyObject *args);
#if defined(LIBXML_SAX1_ENABLED)
PyObject * libxml_xmlSAXDefaultVersion(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SAX1_ENABLED) */
PyObject * libxml_xmlSAXParseFile(PyObject *self, PyObject *args);
#if defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_xmlSaveFile(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_OUTPUT_ENABLED) */
#if defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_xmlSaveFileEnc(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_OUTPUT_ENABLED) */
#if defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_xmlSaveFormatFile(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_OUTPUT_ENABLED) */
#if defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_xmlSaveFormatFileEnc(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_OUTPUT_ENABLED) */
PyObject * libxml_xmlSaveUri(PyObject *self, PyObject *args);
#if defined(LIBXML_SCHEMAS_ENABLED)
PyObject * libxml_xmlSchemaCleanupTypes(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SCHEMAS_ENABLED) */
#if defined(LIBXML_SCHEMAS_ENABLED)
PyObject * libxml_xmlSchemaCollapseString(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SCHEMAS_ENABLED) */
#if defined(LIBXML_SCHEMAS_ENABLED) && defined(LIBXML_DEBUG_ENABLED)
PyObject * libxml_xmlSchemaDump(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SCHEMAS_ENABLED) && defined(LIBXML_DEBUG_ENABLED) */
#if defined(LIBXML_SCHEMAS_ENABLED)
PyObject * libxml_xmlSchemaFree(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SCHEMAS_ENABLED) */
#if defined(LIBXML_SCHEMAS_ENABLED)
PyObject * libxml_xmlSchemaFreeParserCtxt(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SCHEMAS_ENABLED) */
#if defined(LIBXML_SCHEMAS_ENABLED)
PyObject * libxml_xmlSchemaInitTypes(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SCHEMAS_ENABLED) */
#if defined(LIBXML_SCHEMAS_ENABLED)
PyObject * libxml_xmlSchemaIsValid(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SCHEMAS_ENABLED) */
#if defined(LIBXML_SCHEMAS_ENABLED)
PyObject * libxml_xmlSchemaNewDocParserCtxt(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SCHEMAS_ENABLED) */
#if defined(LIBXML_SCHEMAS_ENABLED)
PyObject * libxml_xmlSchemaNewMemParserCtxt(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SCHEMAS_ENABLED) */
#if defined(LIBXML_SCHEMAS_ENABLED)
PyObject * libxml_xmlSchemaNewParserCtxt(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SCHEMAS_ENABLED) */
#if defined(LIBXML_SCHEMAS_ENABLED)
PyObject * libxml_xmlSchemaNewValidCtxt(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SCHEMAS_ENABLED) */
#if defined(LIBXML_SCHEMAS_ENABLED)
PyObject * libxml_xmlSchemaParse(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SCHEMAS_ENABLED) */
#if defined(LIBXML_SCHEMAS_ENABLED)
PyObject * libxml_xmlSchemaSetValidOptions(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SCHEMAS_ENABLED) */
#if defined(LIBXML_SCHEMAS_ENABLED)
PyObject * libxml_xmlSchemaValidCtxtGetOptions(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SCHEMAS_ENABLED) */
#if defined(LIBXML_SCHEMAS_ENABLED)
PyObject * libxml_xmlSchemaValidCtxtGetParserCtxt(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SCHEMAS_ENABLED) */
#if defined(LIBXML_SCHEMAS_ENABLED)
PyObject * libxml_xmlSchemaValidateDoc(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SCHEMAS_ENABLED) */
#if defined(LIBXML_SCHEMAS_ENABLED)
PyObject * libxml_xmlSchemaValidateFile(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SCHEMAS_ENABLED) */
#if defined(LIBXML_SCHEMAS_ENABLED)
PyObject * libxml_xmlSchemaValidateOneElement(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SCHEMAS_ENABLED) */
#if defined(LIBXML_SCHEMAS_ENABLED)
PyObject * libxml_xmlSchemaValidateSetFilename(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SCHEMAS_ENABLED) */
#if defined(LIBXML_SCHEMAS_ENABLED)
PyObject * libxml_xmlSchemaWhiteSpaceReplace(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SCHEMAS_ENABLED) */
PyObject * libxml_xmlSearchNs(PyObject *self, PyObject *args);
PyObject * libxml_xmlSearchNsByHref(PyObject *self, PyObject *args);
PyObject * libxml_xmlSetCompressMode(PyObject *self, PyObject *args);
PyObject * libxml_xmlSetDocCompressMode(PyObject *self, PyObject *args);
PyObject * libxml_xmlSetEntityLoader(PyObject *self, PyObject *args);
PyObject * libxml_xmlSetListDoc(PyObject *self, PyObject *args);
PyObject * libxml_xmlSetNs(PyObject *self, PyObject *args);
PyObject * libxml_xmlSetNsProp(PyObject *self, PyObject *args);
PyObject * libxml_xmlSetProp(PyObject *self, PyObject *args);
PyObject * libxml_xmlSetTreeDoc(PyObject *self, PyObject *args);
#if defined(LIBXML_SAX1_ENABLED)
PyObject * libxml_xmlSetupParserForBuffer(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_SAX1_ENABLED) */
PyObject * libxml_xmlSkipBlankChars(PyObject *self, PyObject *args);
PyObject * libxml_xmlStopParser(PyObject *self, PyObject *args);
PyObject * libxml_xmlStrEqual(PyObject *self, PyObject *args);
PyObject * libxml_xmlStrQEqual(PyObject *self, PyObject *args);
PyObject * libxml_xmlStrcasecmp(PyObject *self, PyObject *args);
PyObject * libxml_xmlStrcasestr(PyObject *self, PyObject *args);
PyObject * libxml_xmlStrcat(PyObject *self, PyObject *args);
PyObject * libxml_xmlStrchr(PyObject *self, PyObject *args);
PyObject * libxml_xmlStrcmp(PyObject *self, PyObject *args);
PyObject * libxml_xmlStrdup(PyObject *self, PyObject *args);
PyObject * libxml_xmlStringDecodeEntities(PyObject *self, PyObject *args);
PyObject * libxml_xmlStringGetNodeList(PyObject *self, PyObject *args);
PyObject * libxml_xmlStringLenDecodeEntities(PyObject *self, PyObject *args);
PyObject * libxml_xmlStringLenGetNodeList(PyObject *self, PyObject *args);
PyObject * libxml_xmlStrlen(PyObject *self, PyObject *args);
PyObject * libxml_xmlStrncasecmp(PyObject *self, PyObject *args);
PyObject * libxml_xmlStrncat(PyObject *self, PyObject *args);
PyObject * libxml_xmlStrncatNew(PyObject *self, PyObject *args);
PyObject * libxml_xmlStrncmp(PyObject *self, PyObject *args);
PyObject * libxml_xmlStrndup(PyObject *self, PyObject *args);
PyObject * libxml_xmlStrstr(PyObject *self, PyObject *args);
PyObject * libxml_xmlStrsub(PyObject *self, PyObject *args);
PyObject * libxml_xmlSubstituteEntitiesDefault(PyObject *self, PyObject *args);
PyObject * libxml_xmlSwitchEncodingName(PyObject *self, PyObject *args);
PyObject * libxml_xmlTextConcat(PyObject *self, PyObject *args);
PyObject * libxml_xmlTextMerge(PyObject *self, PyObject *args);
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderAttributeCount(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderByteConsumed(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderClose(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderConstBaseUri(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderConstEncoding(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderConstLocalName(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderConstName(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderConstNamespaceUri(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderConstPrefix(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderConstString(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderConstValue(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderConstXmlLang(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderConstXmlVersion(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderCurrentDoc(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderCurrentNode(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderDepth(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderExpand(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderGetAttribute(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderGetAttributeNo(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderGetAttributeNs(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderGetLastError(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderGetParserColumnNumber(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderGetParserLineNumber(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderGetParserProp(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderGetRemainder(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderHasAttributes(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderHasValue(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderIsDefault(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderIsEmptyElement(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderIsNamespaceDecl(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderIsValid(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderLocatorBaseURI(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderLocatorLineNumber(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderLookupNamespace(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderMoveToAttribute(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderMoveToAttributeNo(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderMoveToAttributeNs(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderMoveToElement(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderMoveToFirstAttribute(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderMoveToNextAttribute(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderNext(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderNextSibling(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderNodeType(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderNormalization(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderPreserve(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderQuoteChar(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderRead(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderReadAttributeValue(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED) && defined(LIBXML_WRITER_ENABLED)
PyObject * libxml_xmlTextReaderReadInnerXml(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) && defined(LIBXML_WRITER_ENABLED) */
#if defined(LIBXML_READER_ENABLED) && defined(LIBXML_WRITER_ENABLED)
PyObject * libxml_xmlTextReaderReadOuterXml(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) && defined(LIBXML_WRITER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderReadState(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderReadString(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED) && defined(LIBXML_RELAXNG_ENABLED)
PyObject * libxml_xmlTextReaderRelaxNGSetSchema(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) && defined(LIBXML_RELAXNG_ENABLED) */
#if defined(LIBXML_READER_ENABLED) && defined(LIBXML_RELAXNG_ENABLED)
PyObject * libxml_xmlTextReaderRelaxNGValidate(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) && defined(LIBXML_RELAXNG_ENABLED) */
#if defined(LIBXML_READER_ENABLED) && defined(LIBXML_RELAXNG_ENABLED)
PyObject * libxml_xmlTextReaderRelaxNGValidateCtxt(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) && defined(LIBXML_RELAXNG_ENABLED) */
#if defined(LIBXML_READER_ENABLED) && defined(LIBXML_SCHEMAS_ENABLED)
PyObject * libxml_xmlTextReaderSchemaValidate(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) && defined(LIBXML_SCHEMAS_ENABLED) */
#if defined(LIBXML_READER_ENABLED) && defined(LIBXML_SCHEMAS_ENABLED)
PyObject * libxml_xmlTextReaderSchemaValidateCtxt(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) && defined(LIBXML_SCHEMAS_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderSetParserProp(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED) && defined(LIBXML_SCHEMAS_ENABLED)
PyObject * libxml_xmlTextReaderSetSchema(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) && defined(LIBXML_SCHEMAS_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderSetup(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
#if defined(LIBXML_READER_ENABLED)
PyObject * libxml_xmlTextReaderStandalone(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_READER_ENABLED) */
PyObject * libxml_xmlThrDefDoValidityCheckingDefaultValue(PyObject *self, PyObject *args);
PyObject * libxml_xmlThrDefGetWarningsDefaultValue(PyObject *self, PyObject *args);
#if defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_xmlThrDefIndentTreeOutput(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_OUTPUT_ENABLED) */
PyObject * libxml_xmlThrDefKeepBlanksDefaultValue(PyObject *self, PyObject *args);
PyObject * libxml_xmlThrDefLineNumbersDefaultValue(PyObject *self, PyObject *args);
PyObject * libxml_xmlThrDefLoadExtDtdDefaultValue(PyObject *self, PyObject *args);
PyObject * libxml_xmlThrDefPedanticParserDefaultValue(PyObject *self, PyObject *args);
#if defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_xmlThrDefSaveNoEmptyTags(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_OUTPUT_ENABLED) */
PyObject * libxml_xmlThrDefSubstituteEntitiesDefaultValue(PyObject *self, PyObject *args);
#if defined(LIBXML_OUTPUT_ENABLED)
PyObject * libxml_xmlThrDefTreeIndentString(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_OUTPUT_ENABLED) */
PyObject * libxml_xmlURIEscape(PyObject *self, PyObject *args);
PyObject * libxml_xmlURIEscapeStr(PyObject *self, PyObject *args);
PyObject * libxml_xmlURIGetAuthority(PyObject *self, PyObject *args);
PyObject * libxml_xmlURIGetFragment(PyObject *self, PyObject *args);
PyObject * libxml_xmlURIGetOpaque(PyObject *self, PyObject *args);
PyObject * libxml_xmlURIGetPath(PyObject *self, PyObject *args);
PyObject * libxml_xmlURIGetPort(PyObject *self, PyObject *args);
PyObject * libxml_xmlURIGetQuery(PyObject *self, PyObject *args);
PyObject * libxml_xmlURIGetQueryRaw(PyObject *self, PyObject *args);
PyObject * libxml_xmlURIGetScheme(PyObject *self, PyObject *args);
PyObject * libxml_xmlURIGetServer(PyObject *self, PyObject *args);
PyObject * libxml_xmlURIGetUser(PyObject *self, PyObject *args);
PyObject * libxml_xmlURISetAuthority(PyObject *self, PyObject *args);
PyObject * libxml_xmlURISetFragment(PyObject *self, PyObject *args);
PyObject * libxml_xmlURISetOpaque(PyObject *self, PyObject *args);
PyObject * libxml_xmlURISetPath(PyObject *self, PyObject *args);
PyObject * libxml_xmlURISetPort(PyObject *self, PyObject *args);
PyObject * libxml_xmlURISetQuery(PyObject *self, PyObject *args);
PyObject * libxml_xmlURISetQueryRaw(PyObject *self, PyObject *args);
PyObject * libxml_xmlURISetScheme(PyObject *self, PyObject *args);
PyObject * libxml_xmlURISetServer(PyObject *self, PyObject *args);
PyObject * libxml_xmlURISetUser(PyObject *self, PyObject *args);
PyObject * libxml_xmlURIUnescapeString(PyObject *self, PyObject *args);
PyObject * libxml_xmlUTF8Charcmp(PyObject *self, PyObject *args);
PyObject * libxml_xmlUTF8Size(PyObject *self, PyObject *args);
PyObject * libxml_xmlUTF8Strlen(PyObject *self, PyObject *args);
PyObject * libxml_xmlUTF8Strloc(PyObject *self, PyObject *args);
PyObject * libxml_xmlUTF8Strndup(PyObject *self, PyObject *args);
PyObject * libxml_xmlUTF8Strpos(PyObject *self, PyObject *args);
PyObject * libxml_xmlUTF8Strsize(PyObject *self, PyObject *args);
PyObject * libxml_xmlUTF8Strsub(PyObject *self, PyObject *args);
PyObject * libxml_xmlUnlinkNode(PyObject *self, PyObject *args);
PyObject * libxml_xmlUnsetNsProp(PyObject *self, PyObject *args);
PyObject * libxml_xmlUnsetProp(PyObject *self, PyObject *args);
#if defined(LIBXML_VALID_ENABLED)
PyObject * libxml_xmlValidCtxtNormalizeAttributeValue(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_VALID_ENABLED) */
#if defined(LIBXML_VALID_ENABLED)
PyObject * libxml_xmlValidNormalizeAttributeValue(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_VALID_ENABLED) */
#if defined(LIBXML_VALID_ENABLED)
PyObject * libxml_xmlValidateDocument(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_VALID_ENABLED) */
#if defined(LIBXML_VALID_ENABLED)
PyObject * libxml_xmlValidateDocumentFinal(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_VALID_ENABLED) */
#if defined(LIBXML_VALID_ENABLED)
PyObject * libxml_xmlValidateDtd(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_VALID_ENABLED) */
#if defined(LIBXML_VALID_ENABLED)
PyObject * libxml_xmlValidateDtdFinal(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_VALID_ENABLED) */
#if defined(LIBXML_VALID_ENABLED)
PyObject * libxml_xmlValidateElement(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_VALID_ENABLED) */
PyObject * libxml_xmlValidateNCName(PyObject *self, PyObject *args);
PyObject * libxml_xmlValidateNMToken(PyObject *self, PyObject *args);
PyObject * libxml_xmlValidateName(PyObject *self, PyObject *args);
#if defined(LIBXML_VALID_ENABLED)
PyObject * libxml_xmlValidateNameValue(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_VALID_ENABLED) */
#if defined(LIBXML_VALID_ENABLED)
PyObject * libxml_xmlValidateNamesValue(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_VALID_ENABLED) */
#if defined(LIBXML_VALID_ENABLED)
PyObject * libxml_xmlValidateNmtokenValue(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_VALID_ENABLED) */
#if defined(LIBXML_VALID_ENABLED)
PyObject * libxml_xmlValidateNmtokensValue(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_VALID_ENABLED) */
#if defined(LIBXML_VALID_ENABLED)
PyObject * libxml_xmlValidateNotationUse(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_VALID_ENABLED) */
#if defined(LIBXML_VALID_ENABLED)
PyObject * libxml_xmlValidateOneAttribute(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_VALID_ENABLED) */
#if defined(LIBXML_VALID_ENABLED)
PyObject * libxml_xmlValidateOneElement(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_VALID_ENABLED) */
#if defined(LIBXML_VALID_ENABLED)
PyObject * libxml_xmlValidateOneNamespace(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_VALID_ENABLED) */
#if defined(LIBXML_VALID_ENABLED) && defined(LIBXML_REGEXP_ENABLED)
PyObject * libxml_xmlValidatePopElement(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_VALID_ENABLED) && defined(LIBXML_REGEXP_ENABLED) */
#if defined(LIBXML_VALID_ENABLED) && defined(LIBXML_REGEXP_ENABLED)
PyObject * libxml_xmlValidatePushCData(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_VALID_ENABLED) && defined(LIBXML_REGEXP_ENABLED) */
#if defined(LIBXML_VALID_ENABLED) && defined(LIBXML_REGEXP_ENABLED)
PyObject * libxml_xmlValidatePushElement(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_VALID_ENABLED) && defined(LIBXML_REGEXP_ENABLED) */
PyObject * libxml_xmlValidateQName(PyObject *self, PyObject *args);
#if defined(LIBXML_VALID_ENABLED)
PyObject * libxml_xmlValidateRoot(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_VALID_ENABLED) */
#if defined(LIBXML_XINCLUDE_ENABLED)
PyObject * libxml_xmlXIncludeProcess(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XINCLUDE_ENABLED) */
#if defined(LIBXML_XINCLUDE_ENABLED)
PyObject * libxml_xmlXIncludeProcessFlags(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XINCLUDE_ENABLED) */
#if defined(LIBXML_XINCLUDE_ENABLED)
PyObject * libxml_xmlXIncludeProcessTree(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XINCLUDE_ENABLED) */
#if defined(LIBXML_XINCLUDE_ENABLED)
PyObject * libxml_xmlXIncludeProcessTreeFlags(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XINCLUDE_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathAddValues(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathBooleanFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathCastBooleanToNumber(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathCastBooleanToString(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathCastNodeToNumber(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathCastNodeToString(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathCastNumberToBoolean(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathCastNumberToString(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathCastStringToBoolean(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathCastStringToNumber(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathCeilingFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathCmpNodes(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathCompareValues(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathConcatFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathContainsFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathContextSetCache(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathCountFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathDivValues(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathEqualValues(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathErr(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathEval(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathEvalExpr(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathEvalExpression(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathFalseFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathFloorFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathFreeContext(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathFreeParserContext(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathGetContextDoc(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathGetContextNode(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathGetContextPosition(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathGetContextSize(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathGetFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathGetFunctionURI(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathIdFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathInit(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathIsInf(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathIsNaN(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathIsNodeType(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathLangFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathLastFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathLocalNameFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathModValues(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathMultValues(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNamespaceURIFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNewBoolean(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNewCString(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNewContext(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNewFloat(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNewNodeSet(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNewParserContext(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNewString(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNewValueTree(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNextAncestor(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNextAncestorOrSelf(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNextAttribute(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNextChild(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNextDescendant(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNextDescendantOrSelf(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNextFollowing(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNextFollowingSibling(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNextNamespace(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNextParent(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNextPreceding(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNextPrecedingSibling(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNextSelf(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNodeEval(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNodeSetFreeNs(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNormalizeFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNotEqualValues(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNotFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNsLookup(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathNumberFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathOrderDocElems(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathParseNCName(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathParseName(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathParserGetContext(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathPopBoolean(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathPopNumber(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathPopString(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathPositionFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathRegisterAllFunctions(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathRegisterNs(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathRegisterVariable(PyObject *self, PyObject *args);
#endif
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathRegisteredFuncsCleanup(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathRegisteredNsCleanup(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathRegisteredVariablesCleanup(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathRoot(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathRoundFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathSetContextDoc(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathSetContextNode(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathStartsWithFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathStringEvalNumber(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathStringFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathStringLengthFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathSubValues(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathSubstringAfterFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathSubstringBeforeFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathSubstringFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathSumFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathTranslateFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathTrueFunction(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathValueFlipSign(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathValuePop(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathVariableLookup(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPathVariableLookupNS(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPATH_ENABLED)
PyObject * libxml_xmlXPatherror(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPATH_ENABLED) */
#if defined(LIBXML_XPTR_ENABLED)
PyObject * libxml_xmlXPtrEval(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPTR_ENABLED) */
#if defined(LIBXML_XPTR_ENABLED)
PyObject * libxml_xmlXPtrNewContext(PyObject *self, PyObject *args);
#endif /* defined(LIBXML_XPTR_ENABLED) */
