#ifndef XML_CATA_H_PRIVATE__
#define XML_CATA_H_PRIVATE__

#ifdef LIBXML_CATALOG_ENABLED

XML_HIDDEN void
xmlInitCatalogInternal(void);
XML_HIDDEN void
xmlCleanupCatalogInternal(void);

#endif /* LIBXML_CATALOG_ENABLED */

#endif /* XML_CATA_H_PRIVATE__ */
